using System;
using System.IO;
using Unity.Collections;
using UnityEngine;

/// <summary>
/// A structure that holds image pixel data and its dimensions.
/// </summary>
[Serializable]
public struct ImageData
{
    /// <summary>
    /// The pixel data of the image in Color32 format.
    /// </summary>
    public NativeArray<Color32> PixelData;

    /// <summary>
    /// The width and height of the image (width, height).
    /// </summary>
    public (int width, int height) Dimensions;

    public bool isRGBA;

    /// <summary>
    /// Initializes a new instance of the <see cref="ImageData"/> struct using existing pixel data and dimensions.
    /// </summary>
    /// <param name="imageData">The pixel data as a NativeArray of Color32.</param>
    /// <param name="widthHeight">A tuple containing the width and height of the image.</param>
    public ImageData(NativeArray<Color32> imageData, (int width, int height) widthHeight, bool isRgba)
    {
        PixelData = imageData;
        Dimensions = widthHeight;
        isRGBA = isRgba;
    }

    /// <summary>
    /// Initializes a new instance of the <see cref="ImageData"/> struct from a Texture2D object.
    /// </summary>
    /// <param name="sourceTexture">The source texture to extract pixel data from.</param>
    public ImageData(Texture2D sourceTexture)
    {
        PixelData = new NativeArray<Color32>(sourceTexture.GetPixelData<Color32>(0), Allocator.Persistent);
        Dimensions = (sourceTexture.width, sourceTexture.height);
        isRGBA = sourceTexture.format == TextureFormat.RGBA32;
    }

    /// <summary>
    /// Gets the width of the image.
    /// </summary>
    public int Width => Dimensions.width;

    /// <summary>
    /// Gets the height of the image.
    /// </summary>
    public int Height => Dimensions.height;

    /// <summary>
    /// Converts the ImageData back into a new Texture2D.
    /// </summary>
    /// <returns>A Texture2D object created from the stored pixel data.</returns>
    public Texture2D ToTexture2D()
    {
        var texture = new Texture2D(Width, Height, TextureFormat.RGBA32, false);
        texture.LoadRawTextureData(PixelData);
        texture.Apply();
        return texture;
    }

    public void ExportPNG(string path)
    {
        Texture2D texture = this.ToTexture2D();
        byte[] data = texture.EncodeToPNG();
        string thePath;

        if (path.StartsWith("Assets/"))
        {
            thePath = path.Substring("Assets/".Length);
        }
        else
        {
            thePath = path;
        }

        thePath.Replace("/", "\\");

        if (data != null)
        {
            System.IO.File.WriteAllBytes(Path.Combine(Application.dataPath, thePath), data);
        }
    }
}
